-- ------------------------------------------------------------------------------------------------
-- Button classes
-- ------------------------------------------------------------------------------------------------

local function L(text) return text end  -- Localization stub

pol_buttonClasses = {}

function Pol_MonitorButtonFactory(...)
	local button = pol_buttonClasses["SecureBase"](...)
	return button
end

function Pol_MonitorButtonTypeRegister(buttonType, buttonClass)
	if pol_buttonClasses[buttonType] ~= nil then
		return
	end
	pol_buttonClasses[buttonType] = buttonClass
end

-- Base -------------------------------------------------------------------------------------------

local ButtonBase = polOO.Class()
Pol_MonitorButtonTypeRegister("Base", ButtonBase)

-- Static members
local allButtonFrames  = {}
local freeButtonFrames = {}
local buttonFrameCount = 0

function ButtonBase:init(...)
	self.frame = self:_CreateFrame()
	self.frame.button = self
end

function ButtonBase:delete()
	self:_ReleaseFrame(self.frame)
	self.frame = nil
end

function ButtonBase:_CreateFrame(frameTemplate)
	frameTemplate = frameTemplate or "ActionButtonTemplate"
	
	for frame, v in pairs(freeButtonFrames[frameTemplate] or {}) do
		freeButtonFrames[frameTemplate][frame] = nil
		return frame
	end

	local frame = CreateFrame("Button", "Pol_MonitorButton"..buttonFrameCount, UIParent, frameTemplate)
	frame.frameTemplate = frameTemplate
	buttonFrameCount = buttonFrameCount + 1
	
	if not allButtonFrames[frameTemplate] then
		allButtonFrames[frameTemplate] = {}
		freeButtonFrames[frameTemplate] = {}
	end

	-- Strip the border from the icon, the frame border is enough
	local icon = getglobal(frame:GetName()..'Icon')
	icon:SetTexCoord(0.06, 0.94, 0.06, 0.94)

	table.insert(allButtonFrames[frameTemplate], frame)
	return frame
end

function ButtonBase:_ReleaseFrame(frame)
	freeButtonFrames[frame.frameTemplate][frame] = true
	getglobal(frame:GetName().."HotKey"):SetText("")
	getglobal(frame:GetName().."Count"):SetText("")
	getglobal(frame:GetName().."Name"):SetText("")
	frame:Hide()
end	

function ButtonBase:GetDescription()
	return L("Basic square button")
end

function ButtonBase:GetUsage()
	return ""
end

function ButtonBase:SetActive(active)
	if active then
		self:Show()
	else
		self:Hide()
	end
end

function ButtonBase:IsSecure()
	return false
end

function ButtonBase:SetSpell(spellName, texturePath)
	if self.frame:GetAttribute("type") ~= "spell" or self.frame:GetAttribute("spell") ~= spellName then
		self.frame:SetAttribute("type", "spell")
		self.frame:SetAttribute("spell", spellName)
	end
	self:SetTexture(texturePath or Pol_Monitor:GetSpellTexture(spellName))
end

function ButtonBase:SetItem(itemName, texturePath)
	if self.frame:GetAttribute("type") ~= "item" or self.frame:GetAttribute("item") ~= itemName then
		self.frame:SetAttribute("type", "item")
		self.frame:SetAttribute("item", itemName)
		self.frame:SetAttribute("slot", nil)
	end
	self:SetTexture(texturePath or Pol_Monitor:GetItemTexture(itemName))
end

function ButtonBase:SetInventorySlot(slotName, texturePath)
	local slotId, defaultTexture = GetInventorySlotInfo(slotName)
	if self.frame:GetAttribute("type") ~= "item" or self.frame:GetAttribute("slot") ~= slotId then
		self.frame:SetAttribute("type", "item")
		self.frame:SetAttribute("slot", slotId)
		self.frame:SetAttribute("item", nil)
	end
	self:SetTexture(texturePath or GetInventoryItemTexture("player", slotId) or defaultTexture)
end

function ButtonBase:SetAction(actionSlot, texturePath)
	if self.frame:GetAttribute("type") ~= "action" or self.frame:GetAttribute("action") ~= actionSlot then
		self.frame:SetAttribute("type", "action")
		self.frame:SetAttribute("action", actionSlot)
	end
	self:SetTexture(texturePath or GetActionTexture(actionSlot))
end

function ButtonBase:SetMacro(macroName, texturePath)
	if self.frame:GetAttribute("type") ~= "macro" or self.frame:GetAttribute("macro") ~= macroName then
		self.frame:SetAttribute("type", "macro")
		self.frame:SetAttribute("macro", macroName)
		self.frame:SetAttribute("macrotext", nil)
	end
	self:SetTexture(texturePath or select(2, GetMacroInfo(macroName)))
end

function ButtonBase:SetMacroText(macroText, texturePath)
	if self.frame:GetAttribute("type") ~= "macro" or self.frame:GetAttribute("macrotext") ~= macroText then
		self.frame:SetAttribute("type", "macro")
		self.frame:SetAttribute("macro", nil)
		self.frame:SetAttribute("macrotext", macroText)
	end
	self:SetTexture(texturePath)
end

function ButtonBase:SetUnit(unit)
	self.frame:SetAttribute("unit", unit or "player")
end

function ButtonBase:GetFrame()
	return self.frame
end

function ButtonBase:EnableMouse(enabled)
	self.frame:EnableMouse(enabled)
end

function ButtonBase:Show()
	self.frame:Show()
end

function ButtonBase:Hide()
	self.frame:Hide()
end

function ButtonBase:GetTexture()
	return getglobal(self.frame:GetName() .. "Icon"):GetTexture()
end

function ButtonBase:SetTexture(texture)
	getglobal(self.frame:GetName() .. "Icon"):SetTexture(texture)
end

function ButtonBase:SetPoint(point, parentFrame, relPoint, offsetX, offsetY)
	self.frame:SetPoint(point, parentFrame, relPoint, offsetX, offsetY)
end

function ButtonBase:ClearAllPoints()
	self.frame:ClearAllPoints()
end

function ButtonBase:GetWidth()
	return self.frame:GetWidth()
end

function ButtonBase:GetHeight()
	return self.frame:GetHeight()
end

function ButtonBase:SetName(text)
	getglobal(self.frame:GetName().."Name"):SetText(text)
end

function ButtonBase:SetHotKey(text)
	getglobal(self.frame:GetName().."HotKey"):SetText(text)
end

function ButtonBase:SetCount(text)
	getglobal(self.frame:GetName().."Count"):SetText(text)
end

function ButtonBase:SetCooldownTimer(start, duration, enable)
	CooldownFrame_SetTimer(getglobal(self.frame:GetName().."Cooldown"), start or 0, duration or 0, enable and 1 or 0)
end

function ButtonBase:IsUsable()
	local isUsable, notEnoughPower, notInRange = true, false, false
	local buttonType = self.frame:GetAttribute("type")
	
	if buttonType == "spell" then
		local name = self.frame:GetAttribute("spell")
		isUsable, notEnoughPower = IsUsableSpell(name)
		
		if SpellHasRange(name) and IsSpellInRange(name) == 0 then
			notInRange = true
		end
	elseif buttonType == "item" then
		local name = self.frame:GetAttribute("item") or self.frame:GetAttribute("slot")
		isUsable, notEnoughPower = IsUsableItem(name)
		
		if ItemHasRange(name) and IsItemInRange(name) == 0 then
			notInRange = true
		end
	elseif buttonType == "action" then
		local name = self.frame:GetAttribute("action")
		isUsable, notEnoughPower = IsUsableAction(name)
		
		if ActionHasRange(name) and ssIsActionInRange(name) == 0 then
			notInRange = true
		end
	end
	
	return (isUsable and not notInRange), notEnoughPower, notInRange
end

function ButtonBase:HasRange()
	local buttonType = self.frame:GetAttribute("type")
	
	if buttonType == "spell" then
		return SpellHasRange(self.frame:GetAttribute("spell"))
	elseif buttonType == "item" then
		return ItemHasRange(self.frame:GetAttribute("item") or self.frame:GetAttribute("slot"))
	elseif buttonType == "action" then
		return ActionHasRange(self.frame:GetAttribute("action"))
	end
	
	return nil
end

function ButtonBase:UsesPower()
	local buttonType = self.frame:GetAttribute("type")

	if buttonType == "spell" then
		local _, _, _, cost = GetSpellInfo(self.frame:GetAttribute("spell"))
		return cost and cost > 0
	elseif buttonType == "item" then
		return nil
	elseif buttonType == "action" then
		local type, actionID = GetActionInfo(self.frame:GetAttribute("action"))
		if type == "spell" then
			local _, _, _, cost = GetSpellInfo(self.frame:GetAttribute("spell"))
			return cost and cost > 0
		end
		return nil
	end
	return nil
end

function ButtonBase:SetColor(r, g, b)
	local name = self.frame:GetName()
	local icon = getglobal(name.."Icon")
	local normalTexture = getglobal(name.."NormalTexture")
	icon:SetVertexColor(r, g, b)
	normalTexture:SetVertexColor(r, g, b)
end

function ButtonBase:SetColorNormal()
	self:SetColor(1.0, 1.0, 1.0)
end

function ButtonBase:SetColorNotUsable()
	self:SetColor(0.6, 0.6, 0.6)
end

function ButtonBase:SetColorNotEnoughPower()
	self:SetColor(0.1, 0.3, 1.0)
end

function ButtonBase:SetColorNotInRange()
	self:SetColor(0.8, 0.1, 0.1)
end


-- SecureBase -------------------------------------------------------------------------------------

local SecureButtonBase = polOO.Class(pol_buttonClasses.Base)
Pol_MonitorButtonTypeRegister("SecureBase", SecureButtonBase)

function SecureButtonBase:init(...)
	pol_buttonClasses.Base.init(self, ...)
	self.active = true
	self.frame:Show()
end

function SecureButtonBase:SetGroup(group)
	self.group = group
	self:SetActive( self.active )	
end

function SecureButtonBase:_CreateFrame(frameTemplate, unit)
	local frame = pol_buttonClasses.Base._CreateFrame(self, frameTemplate or "ActionButtonTemplate, SecureActionButtonTemplate")
	frame:SetAttribute("unit", unit or "player")
	return frame
end

function SecureButtonBase:SetActive(active)
	local alpha = self.group and self.group:GetAlphaA() or 1
	
	if (not active) then
		alpha = self.group and self.group:GetAlpha() or .6
	end

	self.active = active
	self.frame:SetAlpha(alpha)	
end

function SecureButtonBase:IsSecure()
	return true
end


