-- ------------------------------------------------------------------------------------------------
-- Handles a group of Monitors
-- ------------------------------------------------------------------------------------------------

local function L(text) return text end  -- Localization stub

local Group = polOO.Class()

function Pol_MonitorGroupFactory(name, layoutTypeOrProfile, ...)
	local group = Group(name, layoutTypeOrProfile, ...)
	Pol_Monitor:NotifyGroupObservers("create", group)
	return group
end


-- Base -------------------------------------------------------------------------------------------

function Group:init(name, layoutTypeOrProfile, ...)
	self.name = name
	self.monitors = {}
	self.optionsTableMonitors = {
		-- Individual monitors go here
	}
	if type(layoutTypeOrProfile) ~= "table" then
		-- We're being created on the fly by the user
		self.layout = Pol_MonitorLayoutFactory(layoutTypeOrProfile, name, ...)
		self.profile = {layoutProfile=self.layout:GetProfileTable(), monitors={}}
		self.profile.layoutProfile.layoutType = layoutTypeOrProfile
	else
		-- We're being recreated from a stored profile
		self.profile = layoutTypeOrProfile
		self.layout = Pol_MonitorLayoutFactory(self.profile.layoutProfile.layoutType, self.profile.layoutProfile)
		for i, monitorProfile in ipairs(self.profile.monitors) do
			monitor = Pol_MonitorFactory(monitorProfile.monitorType, monitorProfile)
			self:_AddMonitor(monitor)
		end
		self:_RebuildMonitorOptionsTable()
	end
end

function Group:delete()
	Pol_Monitor:NotifyGroupObservers("delete", self)

	self.layout:delete()
	self.layout = nil
	
	for k, monitor in pairs(self.monitors) do
		self.monitors[k] = nil
		monitor:delete()
	end
end

function Group:GetProfileTable()
	return self.profile
end

function Group:ReNameGroup (name)
	self.name = name
	self.optionsTable.name = "|cFF33FF99Group: "..name;
	self.profile.name = name
end

function Group:GetOptionsTable(order, startOrder)
	order, startOrder = order or 100, startOrder or 1
	
	if not self.optionsTable then
		self.optionsTable = {
			type = "group",
			name = "|cFF33FF99Group: "..self.name,
			desc = L("Manage this group"),
			order = order,
			handler = self,
			
			args = {
					
				rename = {
					type = "input",
					name = L("Group Rename:"),
					desc = L("Specify new group name"),
					set = function(info, v) self:ReNameGroup(v) end,
					order = startOrder + 1,
				},
		
				lock = {
					type = "execute",
					name = L("Lock Position"),
					desc = L("Lock this group so it cannot be moved"),
					func = "LockPosition",
					disabled = function() return not self:IsPositionLocked() end,
					hidden = function() return not self:IsPositionLocked() end,
					order = startOrder + 0,
				},
				unlock = {
					type = "execute",
					name = L("Unlock Position"),
					desc = L("Unlock this group so that the group can be dragged around the screen"),
					func = "UnlockPosition",
					disabled = "IsPositionLocked",
					hidden = "IsPositionLocked",
					order = startOrder + 0,
				},
				display = {
					type = "select",
					style = "dropdown",					
					name = L("Group Display"),
					desc = L("Specify when this monitor group will become active.  Buttons will be hidden otherwise"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) return self:GetDisplay() end,
					set = function(info, v) self:SetDisplay(v) end,
					order = startOrder + 2,
				},				
				mouse = {
					type = "toggle",
					name = L("Ignore mouse input"),
					desc = L("If on, monitors in this group will ignore mouse input, otherwise, buttons can be clicked to cast assocated spells, use items, activate macros, etc"),
					get = function(info) return self:GetIgnoreMouseInput() end,
					set = function(info, v) self:SetIgnoreMouseInput(v) end,
					order = startOrder + 3,
				},
				alphaA = {
					type = "range",
					name = L("Active Fade Level"),
					desc = L("Sets the alpha/fade value for active button monitors, where 0 is invisible and 1 is fully visible when inactive"),
					min = 0,
					max = 1,
					step = .05,
					get = function(info) return self:GetAlphaA() end,
					set = function(info, v) self:SetAlphaA(v) end,
					order = startOrder + 4,
				},				
				alpha = {
					type = "range",
					name = L("Inactive Fade Level"),
					desc = L("Sets the alpha/fade value for inactive button monitors, where 0 is invisible and 1 is fully visible when inactive"),
					min = 0,
					max = 1,
					step = .05,
					get = function(info) return self:GetAlpha() end,
					set = function(info, v) self:SetAlpha(v) end,
					order = startOrder + 5,
				},
				monitor = {
					type = "group",
					name = L("Buttons"),
					desc = L("Create, remove, or edit button monitors created within this group"),
					order = startOrder + 5,
					handler = self,
					args = self.optionsTableMonitors,
				},
				
--				layout = {},
			},
		}
	end
	
	-- Monitors
	for monitorType, class in pairs(pol_monitorClasses) do
		self.optionsTable.args.monitor.args[L("add")..monitorType] = {
			type = "input",
			name = L("Add ")..monitorType,
			desc = L("Add a new %s monitor"):format(monitorType),
			get = false,
			set = function(info, ...) self:AddMonitor(monitorType, ...) end,
			usage = class:GetUsage(),
			order = 1,
		}
		for key, value in pairs(class:GetAddOptionsTable()) do
			-- We trust that the monitor doesn't do anything evil..
			self.optionsTable.args.monitor.args[L("add")..monitorType][key] = value
		end
	end
	
	-- Layouts
	self.optionsTable.args.layout = self.layout:GetOptionsTable(startOrder + 5, 10)

	return self.optionsTable
end

function Group:LockPosition()
	self.layout:LockPosition()
end

function Group:UnlockPosition()
	self.layout:UnlockPosition()
end

function Group:IsPositionLocked()
	return self.layout:IsPositionLocked()
end

function Group:GetIgnoreMouseInput()
	return self.profile.ignoreMouseInput
end

function Group:SetIgnoreMouseInput(value)
	self.profile.ignoreMouseInput = value
	
	for i, monitor in ipairs(self.monitors) do
		monitor:GetButton():EnableMouse(not self.profile.ignoreMouseInput)
	end
end

function Group:GetDisplay()

	if self.profile.display == nil or self.profile.display == "" then
		self.profile.display = "always"
	end
		
	return self.profile.display;
end

function Group:SetDisplay(dtype)
	self.profile.display = dtype
	self:RepaintAllButtons()
end

function Group:GetAlphaA()
	return self.profile.alphaA or 1
end

function Group:SetAlphaA (alpha)
	self.profile.alphaA = alpha
	self:RepaintAllButtons()
end

function Group:GetAlpha()
	return self.profile.alpha or 1
end

function Group:SetAlpha(alpha)
	self.profile.alpha = alpha
	self:RepaintAllButtons()
end

function Group:SetLayout(layoutType, ...)
	error("FIXME: Not implemented")
	-- Don't forget to update layout options
end

function Group:GetMonitors()
	local monitors = {}
	for i, monitor in ipairs(self.monitors) do
		table.insert(monitors, monitor)
	end
	return monitors
end

function Group:MoveMonitor(fromPos, toPos)
	assert(fromPos > 0 and fromPos <= #self.monitors, "Trying to move a monitor from an invalid position")
	assert(toPos > 0 and toPos <= #self.monitors, "Trying to move a monitor to an invalid position")
	local temp = self.monitors[fromPos]
	self.monitors[fromPos] = self.monitors[toPos]
	self.monitors[toPos] = temp
	self.profile.monitors[fromPos] = self.monitors[fromPos]:GetProfileTable()
	self.profile.monitors[toPos] = self.monitors[toPos]:GetProfileTable()
	self.layout:MoveButton(fromPos, toPos)
	self:_RebuildMonitorOptionsTable()
	self.optionsTable.args.monitor = self.optionsTable.args.monitor
end

function Group:AddMonitor(monitorType, ...)
	local monitor = Pol_MonitorFactory(monitorType, "SecureBase", ...)
	local monitorProfile = monitor:GetProfileTable()
	monitorProfile.monitorType = monitorType
	table.insert(self.profile.monitors, monitorProfile)
	self:_AddMonitor(monitor)
	self:_RebuildMonitorOptionsTable()
end

function Group:DeleteMonitor(monitor)

	local profileTable = monitor:GetProfileTable()
	
	for i, v in ipairs(self.profile.monitors) do
		if v == profileTable then
			table.remove(self.profile.monitors, i)
			break
		end
	end
	
	self:_DeleteMonitor(monitor)
	self:_RebuildMonitorOptionsTable()
end

function Group:RepaintAllButtons()
	
	for i, monitor in ipairs(self:GetMonitors()) do
		local button = monitor:GetButton()
		
		if (Pol_Monitor:GetFrameState(button.group:GetDisplay())) then
			button:Show()
		else
			button:Hide()
		end
		
		if button.active ~= nil and button.SetActive then
			button:SetActive(button.active)		
		end
	end
end

function Group:_AddMonitor(monitor)

	table.insert(self.monitors, monitor)
	self.layout:AddButton(monitor:GetButton())
	
	local button = monitor:GetButton()
	
	button:SetGroup(self)
	button:EnableMouse(not self.profile.ignoreMouseInput)
	
	Pol_Monitor:NotifyGroupObservers("addmonitor", self, monitor)
end

function Group:_DeleteMonitor(monitor)
	Pol_Monitor:NotifyGroupObservers("delmonitor", self, monitor)

	for i, v in ipairs(self.monitors) do
		if v == monitor then
			table.remove(self.monitors, i)
			break
		end
	end
	
	self.layout:RemoveButton(monitor:GetButton())

	monitor:delete()
end

function Group:_RebuildMonitorOptionsTable()
	local namePrefix = L("monitor")
	local monitorName
	
	for k, v in pairs(self.optionsTableMonitors) do
		if k:find("^"..namePrefix) then
			self.optionsTableMonitors[k] = nil
		end
	end
	
	for i, monitor in ipairs(self.monitors) do
		local order = 100 + i
		local monitorOptions = monitor:GetOptionsTable(order)
		monitorName = L("monitor")..i

		self.optionsTableMonitors[monitorName] = monitorOptions
		monitorOptions.order = order -- Seems we have to set it again to force a gui update
		monitorOptions.args.deleteSeparator = {
			type = "header",
			name = "",
			order = -5,
		}
		
		monitorOptions.args.up = {
			type = "execute",
			name = L("Move up"),
			desc = L("Move this monitor up"),
			func = function() self:MoveMonitor(i, i-1) end,
			disabled = function() return (i <= 1) end,
			width = "normal",
			order = -4,
		}
		
		monitorOptions.args.down = {
			type = "execute",
			name = L("Move down"),
			desc = L("Move this monitor down"),
			func = function() self:MoveMonitor(i, i+1) end,
			width = "normal",
			disabled = function() return (i >= #self.monitors) end,
			order = -3,
		}
		
		monitorOptions.args.deleteSeparator2 = {
			type = "header",
			name = "",
			order = -2,
		}
		
		monitorOptions.args.delete = {
			type = "execute",
			name = L("Delete"),
			desc = L("Delete this monitor"),
			func = function() self:DeleteMonitor(monitor) end,
			confirm = true,
			width = "double",
			confirmText = L("Delete this button monitor?"),
			order = -1,
		}		
	end
end

